/* Radio Paradise Widget
   Copyright 2005-2022 Alexander Thomas
   Version 2.2.4  2022-09-29  Another website update: change the ‘more’ link to point to the
                  what-is-playing URL. Amazingly, the chumby playlist still works FTTB.
   Version 2.2.3  2019-01-31  Update URLs to new website. Now relying on legacy website for playlist.
                  This will be the last update that will die together with the legacy website,
                  unless there proves to be a new equivalent of the chumby.xml file.
   Version 2.2.2  2014-12-19  Remember album cover visibility setting. Subtler clock warning icon.
   Version 2.2.1  2014-12-04  Made album cover toggle more visible. Clock skew tooltip.
   Version 2.2    2014-12-01  Album cover toggle, AJAX, and more robust refreshing.
   Version 2.1,   2014-11-28  Implemented new refresh algorithm HAL9000 would be proud of.
   Version 2.0,   2014-07-19  Switched to chumby.xml playlist.
   Version 1.0.5, 2014-07-04
   Version 1.0.4, 2008-10-26
   Version 1.0.3, 2006-01-07
   Version 1.0,   2005-10-21

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
 
var widgetVersion = "2.2.4";
var debug = false;
var playlistURL = 'https://legacy.radioparadise.com/chumby.xml';
var songLinkBase = 'https://www.radioparadise.com/music/song/';
var playlistLink = 'https://radioparadise.com/music/what-is-playing';

// All time values are in milliseconds.
var currentEpochTime;
// If this contains a time value, a request is in progress
var requestStartTime = 0;
var httpTimeout = 10000;
// Estimated difference between the local clock and RP server clock. Positive means
// our clock leads the RP clock.
var clockSkew;
// Margin-of-error: an estimate of the maximum error on clockSkew.
var moe;
// Stop trying to reduce the MOE when it no longer exceeds this value. There is little
// point in going below 16 seconds as it can take that long for the server to recover
// from all the clients mobbing it at the time of a song refresh. If this threshold is
// too small, the algorithm will never converge.
var minMOE = 16000;
var refreshStepSize;
// The larger this factor, the slower the algorithm will converge, but the smaller, the
// more redundant server requests it does. I aim to halve MOE in every step, but due to
// overhead I need a factor slightly smaller than 0.5 for that.
var stepShrinkFactor = .4375;
// The time to wait until the next auto refresh will happen.
var nextRefreshDelay;
var lastSongID = '';
// Additional delay to compensate for server overload. This is only considered when
// our skew estimate has converged.
var extraDelay;
var lastRefreshScheduled, refreshRetries, numAutoRefreshesNoChange, timedOutTime;
// Part of the ADK (Announcement Detection Kludge): if 0, we did not kludge recently,
// if 1, we just kludged, if 2, we kludged at the previous song change.
var skippedAnnouncement;

var currentTimeout = 0;
var isHidden = false;
var coverShown = false;
var coverURL = "NoCover.png";

var dbgCount = 0;
var debugLastRefreshTime = 0;

if (window.widget) {
	widget.onhide = onHide;
	widget.onshow = onShow;
}


function resetSkewEstimate() {
	// We start by assuming the clocks are perfectly in sync, which should be correct
	// in most situations.
	clockSkew = 0;
	moe = minMOE;
	refreshStepSize = Math.round(minMOE*stepShrinkFactor);
	nextRefreshDelay = 0;
	extraDelay = 0;
	lastRefreshScheduled = 0;
	refreshRetries = 0;
	numAutoRefreshesNoChange = 0;
	skippedAnnouncement = 0;
	timedOutTime = 0;
}

function debugMessUpSkew() {
	clockSkew = 1234567;
	if(Math.random() < .5)
		clockSkew *= -1;
	clearTimeout(currentTimeout);
	currentTimeout = setTimeout('refreshPlayList("forced")', 3000);
}

function setup()
{
	new AppleInfoButton(document.getElementById("infoButton"), document.getElementById("front"), "black", "black", showInfo);
	new AppleGlassButton(document.getElementById("doneButton"), "Done", showFront);
	document.getElementById("versionText").innerHTML = "Radio Paradise Widget " + widgetVersion;
	resetSkewEstimate();
	// Start out with a small extra delay so we'll refresh a little later than
	// the RP server tells us to, because refreshing at the exact time often fails
	// due to server overload from all clients hammering it at the same time.
	// Add a little randomness to spread the load, for what it's worth...
	extraDelay = 8000 + Math.round(Math.random()*2000);
	refreshPlayList("forced");
	if(debug) {
		document.getElementById('b0rk').style.display = "block";
		document.getElementById('debugLog').style.display = "block";
	}
	if(window.widget) {
		var prefsCoverShown = widget.preferenceForKey("coverShown");
		if(prefsCoverShown != undefined && prefsCoverShown)
			explode();
	}
	return 0;
}

// Prepare and send the request to fetch a new playlist
function refreshPlayList(mode)
{
	if(requestStartTime) {
		if(mode.indexOf('auto') == 0)
			debugLog("ERROR! refreshPlayList(auto) while request in progress?")
		return;
	}

	if(currentTimeout) {
		clearTimeout(currentTimeout);
		currentTimeout = 0;
	}
	// Do not suspend updates as long as the skew is being estimated, to allow converging in the background
	if(isHidden && moe <= minMOE) {
		lastRefreshScheduled = 0;
		skippedAnnouncement = 0;
		return;
	}

	document.getElementById("reload").src = "Reload-pulse.gif";
	var date = new Date();
	requestStartTime = date.getTime();
	fetchURL(mode, playlistURL);
}

// Process the fetched playlist data
function updatePlayList(mode, data)
{
	// It makes most sense to measure current time after we got the response, because in case of
	// overload, most of the delay will be caused by the request being queued somehow, not in handling it.
	// (cfr. Schroeder et al. 2002, "Web servers under overload: How scheduling can help")
	date = new Date();
	currentEpochTime = date.getTime();
	var requestDuration = currentEpochTime - requestStartTime;
	requestStartTime = 0;

	var playlistHTML;
	if(data == '' || data == null) {
		// This mostly happens while trying to load while the file is updated, and the
		// server is unresponsive from too many requests, so try again a few times.
		if(refreshRetries < 6) {
			// Again, a little randomness to avoid an army of widgets bashing the server at the same moments
			var shortDelay = 2000 + Math.round(Math.random()*2000);
			if(moe <= minMOE)
				extraDelay += shortDelay;
			timedOutTime += requestDuration + shortDelay;
			currentTimeout = setTimeout('refreshPlayList("' + mode + '")', shortDelay);
			debugLog("Timeout? Retry in " + pt(shortDelay) + " sec (" + refreshRetries + ", xd=" + pt(extraDelay) + ")");
			debugMsg("(e) No data, retry in " + pt(shortDelay) + " sec (" + refreshRetries + ", xd=" + pt(extraDelay) + ")");
			refreshRetries++;
			return;
		} else {
			nextRefreshDelay = 90000;
			playlistHTML = 'Error: Could not load data!<br>Either you are not connected,<br>the RP website is down,<br>or there is another problem.';
			resetSkewEstimate();
		}
	} else {
		refreshRetries = 0;
		var parser = new DOMParser();
		// I asked Bill to tag the file with a comment during announcements, but never got an
		// answer. I left this in in case he does it anyway (this would avoid the ADK)
		if(data.indexOf("<!--") > -1) {
			var start = data.indexOf("<!--");
			var stop = data.indexOf("-->", start);
			debugLog("FOUND COMMENT: "+data.substr(start, (stop-start)));
		}

		var xmlData = parser.parseFromString(data, "text/xml");
		if(xmlData == undefined) {
			nextRefreshDelay = 10000;
			playlistHTML = "Error: Could not parse playlist XML";
		} else {
			var errorMsg = setRefreshDelay(xmlData, mode);
			if(errorMsg == '')
				playlistHTML = renderPlaylist(xmlData);
			else if(debug) {
				playlistHTML = "CAUGHT ROGUE XML! Check debug log.";
				debugLog(errorMsg + "\n" + data);
			}
		}
	}
	document.getElementById('reload').src = "Reload.png";

	document.getElementById('playlist').innerHTML = playlistHTML;
	if(coverShown)
		document.getElementById("albumImg").src = coverURL;

	if(moe > minMOE)
		document.getElementById('converging').style.display = "block";
	else
		document.getElementById('converging').style.display = "none";
	if(moe <= minMOE && Math.abs(clockSkew) > 60000) {
		var skewInfo = "Your computer's clock is about " + millisecsToTimeString(Math.abs(clockSkew));
		if(clockSkew > 0)
			skewInfo += " ahead of";
		else
			skewInfo += " behind";
		skewInfo += " the Radio Paradise server clock. The RP server could be wrong, but if you suspect your computer's clock to be the culprit, try configuring a time server.";
		document.getElementById('clockWarning').setAttribute("title", skewInfo);
		document.getElementById('clockWarning').style.display = "block";
	}
	else
		document.getElementById('clockWarning').style.display = "none";

	currentTimeout = setTimeout('refreshPlayList("auto")', nextRefreshDelay);
}

function millisecsToTimeString(timeValue)
{
	var output = "";
	timeValue = Math.round(timeValue/1000);
	var secs = timeValue % 60;
	timeValue = (timeValue-secs)/60;
	var minutes = timeValue % 60;
	var hours = (timeValue-minutes)/60;
	if(hours)
		output += hours + " hours";
	if(minutes) {
		if(output != "")
			output += ", ";
		output += minutes + " minutes";
	}
	if(secs || output == "") {
		if(output != "")
			output += ", ";
		output += secs + " seconds";
	}
	return output;
}

// When looking at this code, one might wonder: “WTF? How can a simple scheduling
// routine be so horribly complicated?” Well, this is the big disadvantage of the
// static Chumby playlist compared to the obsolete CGI script that always returned
// time until the next refresh.
// The Chumby playlist relies on absolute time stamps and the assumption of
// perfectly synchronised clocks. The fact that the RP server tends to timeout
// from overload around song changes, obviously further complicates things.
// And last but not least, the playlist is not updated while Bill makes an
// announcement, and will tell clients to update in the past.
// And don't forget that users can hit the refresh button at any time.
// As icing on the cake, sometimes the chumby.xml contains plain wrong data,
// there's nothing I can do about that. I have seen completely wrong songs at times,
// and also a refresh_time that jumped to the past instead of forward.
//
// I could have simply added a test for out-of-sync clocks and then print an error
// message. However, I absolutely hate software updates that reduce functionality
// compared to previous versions, and I do not want to demand users upgrading to
// version 2 to ensure their clock is perfectly in sync. Moreover, I was plain
// curious whether it was possible to make it work under those circumstances.
// Therefore here I attempt to detect and work around out-of-sync clocks.
//
// This is a very non-trivial problem and the only tractable way to do it is an
// iterative approach. This routine may look a little intimidating, but it all
// makes sense when drawing it in a timeline. What it basically does while it tries
// to estimate clock skew, is to try to reload the playlist right before it changes,
// and then after it has changed. The shorter the time between these two refreshes,
// the smaller the margin-of-error on our skew estimate. When the MOE is small
// enough, we try to only refresh once after each song change.
//
// After testing this thoroughly, once in a while I would still see the widget
// re-estimating and settling on a consistent skew. And you know what? This is
// because the RP server effectively jumps forward and back in time now and then,
// I have no idea why. But it makes all my headache-inducing work more worthwhile,
// because the widget is robust against these jumps.
function setRefreshDelay(xml, mode)
{
	dbgCount++;
	var refreshElement = xml.getElementsByTagName("refresh_time");
	var timestampElements = xml.getElementsByTagName("timestamp");
	var songidElements = xml.getElementsByTagName("songid");
	if(refreshElement == undefined || refreshElement.length == 0 || songidElements == undefined || songidElements.length == 0) {
		nextRefreshDelay = 5000;
		// I have seen this happen long ago while Bill made an announcement, although
		// I haven't been able to reproduce it lately. It would actually be very useful
		// if the playlist would contain a hint that an announcement is in progress
		// (cfr. below)
		debugMsg("NO REFRESH IN XML?!");
		// Reset this to avoid messing up everything.
		lastRefreshScheduled = 0;
		return "Error: no refresh_time and/or songid elements in XML\n" + 
		  "moe:"+pt(moe)+", st:"+pt(refreshStepSize)+", nx:"+pt(nextRefreshDelay)+", sk:"+pt(clockSkew)+", xd:"+pt(extraDelay);
	}
	var nextRefreshTime = (refreshElement[0].childNodes[0].nodeValue)*1000;
	var songStartTime = (timestampElements[0].childNodes[0].nodeValue)*1000;
	var songLength = nextRefreshTime-songStartTime;
	var songID = songidElements[0].childNodes[0].nodeValue;

	var dbgMsg = '';
	var currentTimeEstimateRP = currentEpochTime - clockSkew;
	var timeWaitedSinceRefresh = 0;
	if(lastRefreshScheduled)
		timeWaitedSinceRefresh = currentEpochTime - lastRefreshScheduled;
	var autoRefresh = (mode.indexOf('auto') == 0);
	var nextRefreshInThePast = currentTimeEstimateRP - nextRefreshTime;

	if(debug) {
		// Sanity check. This confirmed my suspicion that it was a bad idea to rely on refresh_time to detect song changes.
		if(lastSongID != songID && debugLastRefreshTime == nextRefreshTime)
			debugLog("WTF: songID ("+ lastSongID +"≠"+ songID +") differs but refreshTime is same!");
		else if(lastSongID == songID && debugLastRefreshTime != nextRefreshTime)
			debugLog("WTF: songID is same but refreshTime differs! "+ debugLastRefreshTime/1000 +"≠"+ nextRefreshTime/1000);
	}

	// Yet another complication! When Bill makes an announcement, the playlist freezes and
	// becomes stale, and nextRefreshTime is in the past even with perfectly synchronised clocks.
	// There is no direct way to see if an announcement is in progress, therefore I do it through
	// the following disgusting Announcement Detection Kludge. Only when:
	//  1. we have previously converged, and
	//  2. the song did not change since the previous refresh, and
	//  3. the refresh time is more than refreshStepSize in the past, and
	//  4. it is less than 75 seconds in the past, and
	//  5. we didn't experience this same situation at the previous song change:
	// THEN we assume an announcement is in progress, and retry in a short while. Whew!
	if(moe <= minMOE &&
	   songID == lastSongID &&
	   nextRefreshInThePast > refreshStepSize &&
	   nextRefreshInThePast < 75000 &&
	   skippedAnnouncement != 2) {
		nextRefreshDelay = 4000 + Math.round(Math.random()*2000);
		debugMsg("Announce? " + skippedAnnouncement + ", " + pt(nextRefreshDelay));
		debugLog("Announce " + skippedAnnouncement + " ∆=" + pt(nextRefreshInThePast) +"; +"+ pt(nextRefreshDelay) + ", s="+ clockSkew);
		// There is no reliable way to keep track of this value in this situation, so clear it.
		lastRefreshScheduled = 0;
		skippedAnnouncement = 1;
		return '';
	}

	// Allow an extra error of httpTimeout for the refresh time, because we might have received
	// an outdated file due to server overload.
	if(nextRefreshInThePast > moe + httpTimeout || songStartTime - currentTimeEstimateRP > moe) {
		// B0rk: clocks are severely out-of-sync. Re-initialise. This assumes we're at 75% of the song.
		if(debug) {
			var reason = " is ahead of nextRefreshTime by " + nextRefreshInThePast + " > MOE+timeout="+ (moe+httpTimeout);
			if(songStartTime - currentTimeEstimateRP > moe)
				reason = " is behind songStartTime by " + (songStartTime-currentTimeEstimateRP) + " > MOE="+moe;
			debugLog("RESET because time estimate (sk="+clockSkew+")" + reason + ". sA="+skippedAnnouncement);
		}
		moe = songLength;
		refreshStepSize = Math.round(moe/4);
		nextRefreshDelay = refreshStepSize;
		// Ignore this until we have converged, otherwise things only get more complicated
		extraDelay = 0;
		clockSkew = currentEpochTime + nextRefreshDelay - nextRefreshTime;
		numAutoRefreshesNoChange = 0;
		skippedAnnouncement = 0;
		timedOutTime = 0;
		dbgMsg = '(a) ';
	}
	else if(songID == lastSongID) {
		if(autoRefresh) {
			// We refreshed too early. Schedule a new refresh within reasonable time.
			if(numAutoRefreshesNoChange > 5) {
				// If this is happening too often in a row, increase the MOE and step size.
				moe = Math.round(moe*1.2);
				refreshStepSize = Math.round(refreshStepSize*1.2);
			}
			nextRefreshDelay = refreshStepSize;
			clockSkew = currentEpochTime + nextRefreshDelay - nextRefreshTime;
			numAutoRefreshesNoChange++;
			dbgMsg = '(b-'+numAutoRefreshesNoChange+') ';
		} else {
			// This situation gives no additional information. Just reschedule refresh at its original time.
			nextRefreshDelay = nextRefreshDelay - timeWaitedSinceRefresh;
			dbgMsg = '(Fb) ';
		}
	}
	else { // The song has changed.
		numAutoRefreshesNoChange = 0;
		if(skippedAnnouncement == 1)
			skippedAnnouncement = 2;
		else
			skippedAnnouncement = 0;

		if(moe > minMOE) {
			// The time between two refreshes where the last one reveals a song change,
			// is a good estimate for the MOE.
			if(timeWaitedSinceRefresh && timeWaitedSinceRefresh < moe)
				moe = timeWaitedSinceRefresh;
			// Do not make MOE too small, otherwise the b0rk-detector might trip too easily.
			if(moe <= minMOE) {
				moe = minMOE;
				debugLog("Converged!");
			}
			refreshStepSize = Math.round(moe*stepShrinkFactor);
			
			if(autoRefresh)
				dbgMsg = '(c) ';
			else
				dbgMsg = '(Fc) ';
			nextRefreshDelay = refreshStepSize;
			// We can be pretty certain that there is no use in refreshing sooner than this:
			var optimalRefreshDelay = songLength - moe - nextRefreshDelay - httpTimeout - timedOutTime;
			if(optimalRefreshDelay > nextRefreshDelay)
				nextRefreshDelay = optimalRefreshDelay;
			clockSkew = currentEpochTime + nextRefreshDelay - nextRefreshTime;
			if(moe <= minMOE)
				clockSkew += refreshStepSize;
		}
		else { // Our skew estimate is good enough, so we can rely on the skew-adjusted refresh
			// time and politely do (ideally) only one refresh per song.
			// To cope with possible drift, slightly shrink the skew. If it becomes too small,
			// it will be corrected by (b) at the next refresh. Don't do this however
			// if the server is too unresponsive, otherwise the skew can get out of control.
			var excess = 0;
			if(extraDelay < minMOE)
				excess = 250;
			if(autoRefresh)
				dbgMsg = '(d) ';
			else { // Forced refresh
				if(lastRefreshScheduled && timeWaitedSinceRefresh < (nextRefreshDelay - extraDelay)) {
					// We had an auto-refresh planned and the song proves to have changed
					// earlier than we planned our refresh even considering the extra delay,
					// so we can subtract the excess time from our skew.
					excess = nextRefreshDelay - extraDelay - timeWaitedSinceRefresh;
				}
				dbgMsg = '(Fd) ';
			}
			clockSkew -= excess;
			currentTimeEstimateRP += excess;
			// Shrink the extra delay as well, in the hopes that the server becomes less overloaded.
			extraDelay *= .96;
			nextRefreshDelay = nextRefreshTime - currentTimeEstimateRP + extraDelay;
		}
	}
	
	timedOutTime = 0;
	lastSongID = songID;
	debugLastRefreshTime = nextRefreshTime; //DEBUG
	lastRefreshScheduled = currentEpochTime;
	debugMsg(dbgMsg + "moe:"+pt(moe)+", st:"+pt(refreshStepSize)+", nx:"+pt(nextRefreshDelay)+", sk:"+pt(clockSkew)+", xd:"+pt(extraDelay));
	return '';
}

function pt(msecs)
{
	return (Math.round(msecs/100))/10;
}
	
function renderPlaylist(xml)
{
	var artistElements = xml.getElementsByTagName("artist");
	var titleElements = xml.getElementsByTagName("title");
	var albumElements = xml.getElementsByTagName("album");
	var songidElements = xml.getElementsByTagName("songid");
	var dateElements = xml.getElementsByTagName("release_date");
	var artElements = xml.getElementsByTagName("coverart");

	var output = '';
	for(i=0; i<artistElements.length; i++) {
		var artist = artistElements[i].childNodes[0].nodeValue;
		var title = titleElements[i].childNodes[0].nodeValue;
		var album = '?';
		var releaseDate;
		if(albumElements[i].childNodes[0] != undefined) {
			album = albumElements[i].childNodes[0].nodeValue.replace(/&/g, "&amp;");;
			album = album.replace(/'/g, "&#39;");
		}
		if(dateElements[i].childNodes[0] != undefined) {
			releaseDate = dateElements[i].childNodes[0].nodeValue;
			if(releaseDate != '')
				album += " (" + releaseDate + ")";
		}

		// Try to cut off too long entries in a somewhat smart way. Of course this
		// totally disregards font metrics but it is better than nothing.
		if(artist.length + title.length > 45) {
			if(title.length < 24)
				artist = artist.substr(0, 44-title.length) + '…';
			else if(artist.length < 23)
				title = title.substr(0, 44-artist.length) + '…';
			else {
				artist = artist.substr(0, 21) + '…';
				title = title.substr(0, 22) + '…';
			}
		}
		var url = songLinkBase + songidElements[i].childNodes[0].nodeValue;
		var xtraStyle = '';
		if(i < 3)
			xtraStyle = " class='song" + i + "'";
		else
			xtraStyle = " class='song3'";
		output += "<a href='#'" + xtraStyle + " onClick='widget.openURL(\"" + url + "\")' title='" + album + "'>"
		       + artist + " - " + title + "</a><br>";
	}
	
	output += "<div id='moreLink'><a href='#' onClick='widget.openURL(\"" + playlistLink + "\")'>… more</a></div>";
	
	coverURL == '';
	if(artElements[0].childNodes[0] != undefined) {
		coverURL = artElements[0].childNodes[0].nodeValue;
		// there are three sizes of covers, 'covers/s/': 70x70 images, m: 160x160, l: various larger sizes
		coverURL = coverURL.replace(/covers\/.\//, "covers/s/");
	}
	if(coverURL == '')
		coverURL = "NoCover.png";
	return output;
}

function fetchURL(mode, url)
{
	try {
		var xml_request = new XMLHttpRequest();
		// At last, I switched to async requests, which was actually trivial.
		// The advantages are that it does not block the entire Dashboard while the request is
		// in progress (oops), and that any other operations can keep on running in the meantime
		// (e.g. the pulsating refresh icon).
		xml_request.open("GET", url, true);
		xml_request.setRequestHeader("Cache-Control", "no-cache");
		// At last, with the async requests I can enforce this! Still, I use the same value as for sync requests.
		xml_request.timeout = httpTimeout;
		xml_request.onreadystatechange = function() {
			if(xml_request.readyState == 4) {
				if(xml_request.status == 200)
					updatePlayList(mode, xml_request.responseText);
				else {
					debugLog("HTTP status "+xml_request.status);
					updatePlayList(mode, '');
				}
			}
		}
		xml_request.send(null);
	}
	catch(e) {
		debugMsg(e.message);
		debugLog("HTTP EXCEPT! "+e.message);
	}
}

function goToRP()
{
	var url = 'http://www.radioparadise.com/';
	if(window.widget)
		widget.openURL(url);
	else
		document.location.href = url;
}

function goToLex()
{
	var url = 'http://www.dr-lex.be/software/';
	if(window.widget)
		widget.openURL(url);
	else
		document.location.href = url;
}

function onHide () {
	isHidden = true;
}

function onShow() {
	isHidden = false;
	if( !currentTimeout )
		refreshPlayList("forced");
}

function showInfo() {
	if(window.widget)
		widget.prepareForTransition("ToBack");

	front.style.display = "none";
	back.style.display  = "block";
	
	if(window.widget)
		setTimeout('widget.performTransition();', 0);
}

function showFront() {
	if(window.widget)
		widget.prepareForTransition("ToFront");
 
		back.style.display  = "none";
		front.style.display = "block";
 
	if(window.widget)
		setTimeout('widget.performTransition();', 0);
}

function debugMsg(message) {
	if(debug)
		document.getElementById('debug').innerHTML = message;
}

function debugLog(message) {
	if(debug)
		document.getElementById("debugLog").value += "["+dbgCount+"]" + message + "\n";
}

function explode() {
	if(coverShown)
		return;
	document.getElementById("frontImg").src = "Wide.png";
	document.getElementById("expander").style.visibility = "hidden";
	//window.resizeTo(360, 106);
	var animator = new AppleAnimator(500, 30, 281, 360, function(animation, animationValue, start, finish) {
		window.resizeTo(Math.round(animationValue), 106);
		if(finish) {
			document.getElementById("albumImg").src = coverURL;
			document.getElementById("albumCover").style.visibility = "visible";
			coverShown = true;
			widget.setPreferenceForKey(coverShown, "coverShown");
		}
	});
	animator.start();
}

function implode() {
	if(!coverShown)
		return;
	document.getElementById("albumCover").style.visibility = "hidden";
	//window.resizeTo(281, 106);
	var animator = new AppleAnimator(500, 30, 360, 281, function(animation, animationValue, start, finish) {
		window.resizeTo(Math.round(animationValue), 106);
		if(finish) {
			document.getElementById("frontImg").src = "Default.png";
			document.getElementById("expander").style.visibility = "visible";
			coverShown = false;
			widget.setPreferenceForKey(coverShown, "coverShown");
		}
	});
	animator.start();
}